
#include "CAN.h"
#include "usb_lib.h"
#include "hw_config.h"
#include "usb_wrapper.h"

callback_fun_type can_callback_table;
volatile  uint8_t usb_can_flag;

#ifdef __cplusplus
extern "C" {
#endif

void USB_LP_CAN1_RX0_IRQHandler(void)
{
    if(usb_can_flag == 1)
	{
		usb_callback();
	}
	else
	{
		can_callback_table();
		CAN_FIFORelease(CAN1, CAN_FIFO0);	
	}

}


#ifdef __cplusplus
}
#endif

void CAN::attach_interrupt(void (*callback_fun)(void))
{
    can_callback_table = callback_fun;
}

void CAN::interrupt(FunctionalState enable)
{
    if(enable == ENABLE)
    {
        CAN_ITConfig(CAN1, CAN_IT_FMP0, ENABLE);
    }
    else
    {
        CAN_ITConfig(CAN1, CAN_IT_FMP0, DISABLE);
    }
}

CAN::CAN(GPIO *p_pin_rx, GPIO *p_pin_tx)
{
    this->pin_rx = p_pin_rx;
    this->pin_tx = p_pin_tx;
}


const uint16_t CANBAUD[][4] =
{
    { CAN_SJW_1tq, CAN_BS1_6tq, CAN_BS2_3tq, 108 },     //33k
    { CAN_SJW_1tq, CAN_BS1_6tq, CAN_BS2_3tq, 72 },      //50k
    { CAN_SJW_1tq, CAN_BS1_3tq, CAN_BS2_5tq, 64 },      //62.5k
    { CAN_SJW_1tq, CAN_BS1_6tq, CAN_BS2_8tq, 25 },      //96k
    { CAN_SJW_1tq, CAN_BS1_6tq, CAN_BS2_3tq, 36 },      //100k
    { CAN_SJW_1tq, CAN_BS1_3tq, CAN_BS2_5tq, 32 },      //125k
    { CAN_SJW_1tq, CAN_BS1_3tq, CAN_BS2_5tq, 16 },      //250k
    { CAN_SJW_1tq, CAN_BS1_3tq, CAN_BS2_5tq, 8 },       //500k
    { CAN_SJW_1tq, CAN_BS1_9tq, CAN_BS2_5tq, 3 },       //800k
    { CAN_SJW_1tq, CAN_BS1_3tq, CAN_BS2_5tq, 4 },       //1M
};

void CAN::set_bps(BSP_CAN_BAUD bps)
{
    CAN_InitTypeDef CAN_InitStructure;

    CAN_InitStructure.CAN_SJW = CANBAUD[bps][0];//CAN_SJW_1tq;
    CAN_InitStructure.CAN_BS1 = CANBAUD[bps][1];//CAN_BS1_6tq;
    CAN_InitStructure.CAN_BS2 = CANBAUD[bps][2];//CAN_BS2_8tq;
    CAN_InitStructure.CAN_Prescaler = CANBAUD[bps][3];//60;

    CAN_InitStructure.CAN_TTCM = DISABLE;   //ʱ䴥
    CAN_InitStructure.CAN_ABOM = ENABLE;    //Զ߹
    CAN_InitStructure.CAN_AWUM = ENABLE;    //Զ
    CAN_InitStructure.CAN_NART = ENABLE;    //Զش
    CAN_InitStructure.CAN_RFLM = DISABLE;
    CAN_InitStructure.CAN_TXFP = DISABLE;
    CAN_InitStructure.CAN_Mode = CAN_Mode_Normal;

    CAN_Init(CAN1, &CAN_InitStructure);
}


void CAN::set_filter_idmask(u8 nCanType,u8 num,u32 ID,u32 mask)
{
		CAN_FilterInitTypeDef  CAN_FilterInitStructure;

		CAN_FilterInitStructure.CAN_FilterNumber=num;						
		CAN_FilterInitStructure.CAN_FilterMode=CAN_FilterMode_IdMask;		
    CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
	
		if(nCanType == CAN_ID_STD)
		{/* std id  */
			CAN_FilterInitStructure.CAN_FilterIdHigh= ID<<5;
			CAN_FilterInitStructure.CAN_FilterIdLow=0|CAN_ID_STD;
		}
		else
		{
		/* ext id  */
			CAN_FilterInitStructure.CAN_FilterIdHigh= ((ID<<3)>>16) & 0xffff;
			CAN_FilterInitStructure.CAN_FilterIdLow= ((ID<<3)& 0xffff) | CAN_ID_EXT;
		}
		
		CAN_FilterInitStructure.CAN_FilterMaskIdHigh=(mask>>16)&0xffff;
		CAN_FilterInitStructure.CAN_FilterMaskIdLow=mask&0xffff;
		
		CAN_FilterInitStructure.CAN_FilterFIFOAssignment=CAN_Filter_FIFO0 ;
		CAN_FilterInitStructure.CAN_FilterActivation=ENABLE;
		CAN_FilterInit(&CAN_FilterInitStructure);
}


void CAN::set_filter_idlist(u8 nCanType,u8 num,u32 ID)
{
		CAN_FilterInitTypeDef  CAN_FilterInitStructure;

		CAN_FilterInitStructure.CAN_FilterNumber=num;						
		CAN_FilterInitStructure.CAN_FilterMode=CAN_FilterMode_IdList;	
		CAN_FilterInitStructure.CAN_FilterScale=CAN_FilterScale_32bit;


		
		if(nCanType == CAN_ID_STD)
		{/* std id  */
			CAN_FilterInitStructure.CAN_FilterIdHigh= ID<<5;	
			CAN_FilterInitStructure.CAN_FilterIdLow= 0|CAN_ID_STD;
		}
		else
		{
			/* ext id  */
			CAN_FilterInitStructure.CAN_FilterIdHigh= ((ID<<3)>>16) & 0xffff;
			CAN_FilterInitStructure.CAN_FilterIdLow= ((ID<<3)& 0xffff) | CAN_ID_EXT;				
		}
		
		CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0;			//????16???????
    CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0;			//????16???????

		CAN_FilterInitStructure.CAN_FilterFIFOAssignment=CAN_Filter_FIFO0 ;
		CAN_FilterInitStructure.CAN_FilterActivation=ENABLE;
		CAN_FilterInit(&CAN_FilterInitStructure);
}

void CAN::begin(BSP_CAN_BAUD bps)
{

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1 | RCC_APB1Periph_CAN2, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

    /*to do */
    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    pin_rx->mode(INPUT_PU);
    pin_tx->mode(AF_PP);



    NVIC_InitTypeDef NVIC_InitStructure;
    NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;	   //CAN1 RX0??
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;		   //?????0
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;			   //?????0
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    set_bps(bps);
	
		for(u8 i=0;i<3;i++)CAN_CancelTransmit(CAN1,i);

}

u8 CAN::write(CanTxMsg *pCanMsg)
{
    u32 nTime = 0x3fff;
    u8 TMailbox;

    TMailbox = CAN_Transmit(CAN1, pCanMsg);

    while((CAN_TransmitStatus(CAN1, TMailbox) != CANTXOK) && (--nTime));

    if(nTime == 0)
    {
        CAN_CancelTransmit(CAN1, TMailbox);//ʹʱȡ
        return false;
    }
    else
    {
        return true;
    }
}

void CAN::read(CanRxMsg *pCanMsg)
{
    CAN_Receive(CAN1, CAN_FIFO0, pCanMsg );
}

u8 CAN::available(void)
{
    return CAN_MessagePending(CAN1, CAN_FIFO0);
}

