/*
file   : lcd1602.cpp
author : shentq
version: V1.0
date   : 2015/7/5

Copyright 2015 shentq. All Rights Reserved.

Copyright Notice
No part of this software may be used for any commercial activities by any form or means, without the prior written consent of shentqlf@163.com.

Disclaimer
This specification is preliminary and is subject to change at any time without notice. shentqlf@163.com assumes no responsibility for any errors contained herein.
*/

#include "lcd1602.h"
#include "stdio.h"

#define DATAOUT  GPIOB->ODR             //PD[0..7]--(LCD)D0~D7

LCD1602::LCD1602(GPIO *led,GPIO *en,GPIO *rw,GPIO *rs,GPIO *d0,GPIO *d1,GPIO *d2,GPIO *d3,GPIO *d4,GPIO *d5,GPIO *d6,GPIO *d7)
{
    this->led   = led;
    this->en	= en;
    this->rw	= rw;
    this->rs	= rs;
    this->d0    = d0;
    this->d1	= d1;
    this->d2	= d2;
    this->d3	= d3;
    this->d4	= d4;
    this->d5	= d5;
    this->d6	= d6;
    this->d7	= d7;

    delay_times = 1;
}

inline void LCD1602::lcd_delay(uint32_t nCount)
{
   for(; nCount != 0; nCount--);
}
inline uint8_t LCD1602::bz(void)
{                         
	uint8_t result;


	d7->mode(INPUT);
	rs->reset();
	rw->set();
	en->set();
	lcd_delay(delay_times);
	result = d7->read();//
	en->reset();

	return result; 

}
/******************************д*************************************************/	
/*********************************************************************
*    ƣWritecmd()
*    ܣдָLCD1602,ָռһֽ
*ڲcmd:дָ
*ڲ
*********************************************************************/
void LCD1602::wcmd(uint8_t cmd)
{                          
	while(bz())
	{
		;
	}

	d7->mode(OUTPUT_PP);
	rs->reset();
	rw->reset();
	en->reset();
	
	d0->write(cmd&0x01);
	d1->write(cmd&0x02);
	d2->write(cmd&0x04);
	d3->write(cmd&0x08);
	d4->write(cmd&0x10);
	d5->write(cmd&0x20);
	d6->write(cmd&0x40);
	d7->write(cmd&0x80);
//Ӳʱ˳Ľʹ·ʽ
//	DATAOUT &= 0XFF00;
//	DATAOUT |= cmd;


	lcd_delay(delay_times);
	en->set();
	lcd_delay(delay_times);
	en->reset();
	lcd_delay(delay_times);
}	
/*********************************************************************
*    ƣWritedata()
*    ܣдһֽݵLCD1602
*ڲdat޷ֽͣ0~255  ASCIIַ
*ڲ
*********************************************************************/
void LCD1602::wdat(uint8_t dat) 
{                         
	while(bz())//ȴLCD1602
	{
		;
	}

	d7->mode(OUTPUT_PP);
	rs->set();
	rw->reset();
	en->reset();

	d0->write(dat&0x01);
	d1->write(dat&0x02);
	d2->write(dat&0x04);
	d3->write(dat&0x08);
	d4->write(dat&0x10);
	d5->write(dat&0x20);
	d6->write(dat&0x40);
	d7->write(dat&0x80);

	
//Ӳʱ˳Ľʹ·ʽ
//	DATAOUT &= 0XFF00;
//	DATAOUT |= dat;

	lcd_delay(delay_times);
	en->set();
	lcd_delay(delay_times);
	en->reset();
}
	
/*********************************************************************
*    ƣpos()
*    ܣ趨ʾλ
*ڲpos:ʾλ,ֵķΧ£
 *   0x00----------------0x0f 0x10-------0x27 һУһʾ16ַ
 *   0x40----------------0x4f 0x50-------0x67 ڶ
*ڲ
*********************************************************************/
void LCD1602::pos(uint8_t pos)
{                         
	wcmd(pos | 0x80);
}

/*********************************************************************
*    ƣsetPos()
*    ܣϰ趨ʾλ
*ڲrow:Уrow=1ʾһУrow=2ʾڶ
 *         col:У0~15ָʾУΧ0~40
*ڲ
*********************************************************************/
void LCD1602::set_pos(uint8_t row,uint8_t col)
{
	if(row==1) wcmd(col | 0x80);
	else wcmd(col | 0xC0);
}
/*********************************************************************
*  ܣʾһַ	  
*  ڣchʾַ
*********************************************************************/
void LCD1602::disp_char(char ch)
{
	wdat(ch);
}
/*********************************************************************
*  ܣָλʾһַ	  
*  ڣrow: 12        col:У0~15
 *       chʾַ
*********************************************************************/
void LCD1602::disp_char(uint8_t row,uint8_t col,char ch)
{
	set_pos(row,col);
	wdat(ch);
}
/*********************************************************************
*    ƣdispString()
*    ܣʹLCD1602ʾһַʾλǰ趨
*ڲstr[]:ʾַ
*ڲ
*********************************************************************/
void LCD1602::disp_string(char str[])
{
    uint8_t i=0;
	while(str[i] != '\0')
	{
	   wdat(str[i]);
	   ++i;
	}
}
/*********************************************************************
*    ƣsetPos_dispString()
*    ܣʹLCD1602ָλÿʼʾһַ
*ڲrow:ʾУ12
 *         col:ʾУ0~15 0~40 
 *         str[]:ʾַ
*ڲ
*˵    ָλʾĳʼλãһַʾλ
*********************************************************************/
void LCD1602::disp_string(uint8_t row,uint8_t col,char str[])
{
	set_pos(row,col);
	disp_string(str);
}
void LCD1602::printf(const char *fmt,...)
{
	char buf[16];
	u8 i = 0;
	va_list va_params;   
	va_start(va_params,fmt);   
	vsprintf(buf,fmt,va_params);   
	va_end(va_params); 
	
	while(buf[i] != '\0')
	{
	   wdat(buf[i++]);
	}
}
void LCD1602::printf(uint8_t row,uint8_t col,const char *fmt,...)
{
	char buf[16];
	u8 i = 0;
	va_list va_params;   
	va_start(va_params,fmt);   
	vsprintf(buf,fmt,va_params);   
	va_end(va_params); 
	set_pos(row,col);
	while(buf[i] != '\0')
	{
	   wdat(buf[i++]);
	}
}

/*********************************************************************
*    ƣDispDateTime()
*    ܣʾʾʱ
*ڲyear:,ֵ
 *         month:£1~12
 *         day:գ1~31
 *         hour:ʱ1~24   1~12
 *         min:֣0~59
 *         sec:룬0~59
*ڲ
*˵    ʹǰȺ궨USE_DISP_DATEڰļ֮ǰЧ
*ʾʽDate:yyyy-mm-dd  *
 *         Time:hh:mm:ss    *
*********************************************************************/

void LCD1602::disp_date_time(uint32_t year,uint8_t month,uint8_t day,uint8_t hour,uint8_t min,uint8_t sec)
{
	char s[8];
	set_pos(1,0);
	disp_string("Date:");
	
	sprintf(s,"%04d-%02d-%02d",year,month,day);
	disp_string(s);
	

	set_pos(1,15);
	disp_char('*'); //һнʾ
	set_pos(2,0);
	disp_string("Time:  ");

	sprintf(s,"%02d:%02d:%02d",hour,min,sec);
	disp_string(s);

	set_pos(2,15);
	disp_char('*'); //ڶнʾ
}

void LCD1602::disp_time_HMS(uint8_t hour,uint8_t min,uint8_t sec)
{
	char s[8];

	set_pos(2,0);
	disp_string("Time:  ");

	sprintf(s,"%02d:%02d:%02d",hour,min,sec);
	disp_string(s);

	set_pos(2,15);
	disp_char('*'); //ڶнʾ
}
//Ļһڹʾ
void LCD1602::shift_left()
{
	wcmd(0x18);
}
//Ļһڹʾ
void LCD1602::shift_right(void)
{
	wcmd(0x1C);
}
//ʾ
void LCD1602::clear(void)
{
	wcmd(0x01);
}
//ʹ껹ԭλ
void LCD1602::Return(void)
{
	wcmd(0x02);
}
//ʾ
void LCD1602::close(void)
{
	wcmd(0x08);
}
//ʾ
void LCD1602::open(void)
{
	wcmd(0x0C);
}
/*********************************************************************
*    ƣFlickerChar()
*    ܣʹָλַ˸ʾ
*ڲrow: 1,2  
 *         col: 0~15
*ڲ
*˵    д0x0Dʾ˸д0x0Fʾ˸
 *         һ趨˸󣬻λñ仯˸˸д0x0C
*********************************************************************/
void LCD1602::flicker(void)
{
	wcmd(0x0D);
}
void LCD1602::flicker_char(uint8_t row,uint8_t col)
{
	wcmd(0x0D);
	set_pos(row,col);
}
//رַ˸
void LCD1602::close_flicker(void)
{
	wcmd(0x0C);
}
//Ļ˸һ
void LCD1602::flicker_screen(void)
{
	wcmd(0x08);//ʾ
	lcd_delay(0x3fffff);
	wcmd(0x0C);//ʾ
	lcd_delay(0x3fffff);
}
void LCD1602::back_light(u8 i)
{
	if(i == 1)
		led->set();
	else
		led->reset();
}

/*********************************************************************
*    ƣInit()
*    ܣʼLCD1602
*ڲ
*ڲ
*0x38:16*2ʾ5*&ʾַ8λ;ִָʱ40us
 *         0x0C:ʾع	  40us
 *         0x08:ʾع	  40us
 *         0x0D:ַ˸ع꣬趨λã˸0.4ms
 *         0x0F:ַ˸
 *         0x06:дԶƹ꣬ͨΣʾ  40us
 *         0x04:дԶƹ꣬Դʾ	 40us
 *         0x01:ʾݣ  1.64ms
 *         0x02:ʹ껹ʼλ	  1.64ms
 *         0x18:ĻַͬʱһʺϹʾ 40us
 *         0x1C:ĻַͬʱһʺϹʾ 40us
*********************************************************************/
void LCD1602::begin(void)
{   
	led->mode(OUTPUT_PP);
	en->mode(OUTPUT_PP);
	rw->mode(OUTPUT_PP);
	rs->mode(OUTPUT_PP);
	d0->mode(OUTPUT_PP);
	d1->mode(OUTPUT_PP);
	d2->mode(OUTPUT_PP);
	d3->mode(OUTPUT_PP);
	d4->mode(OUTPUT_PP);
	d5->mode(OUTPUT_PP);
	d6->mode(OUTPUT_PP);
	d7->mode(OUTPUT_PP);

	wcmd(0x38);  //16*2ʾ5*78λ
	lcd_delay(100);
	wcmd(0x38);  //16*2ʾ5*78λ
	lcd_delay(100);
	wcmd(0x38);  //16*2ʾ5*78λ
	lcd_delay(100);
	wcmd(0x08);  //ȹʾʾ
	lcd_delay(100);
	wcmd(0x01);  //LCDʾ
	lcd_delay(100);
	wcmd(0x06);  //Զƹ,0x04Ϊƹ
	lcd_delay(100);
	wcmd(0x0c);  //ʾع;0x08Ϊʾ
	lcd_delay(100);
	back_light(1);
}
void LCD1602::test(void)
{
	char str[]={"Hello World  !"};
	disp_string(1,1,str);
	flicker_char(2,3);
	flicker_screen();
	flicker_screen();
	flicker_screen();
}

