#ifndef __NRF24L01_H
#define __NRF24L01_H
#include "ebox.h"

class NRF24L01
{
    public:
        NRF24L01(GPIO *irq,GPIO *ce,GPIO *cs,SPI *spi);
        void    begin(uint8_t dev_num);
        int     self_check();
    
        void    set_tx_mode();
        void    set_rx_mode();

        int     send(uint8_t *buf,uint8_t length);
        int     recv(uint8_t *buf);
        int     status(void);
        int     read_test(void);


    private:
        GPIO *irq;
        GPIO *ce;
        GPIO *cs;
        SPI *spi;
		SPI_CONFIG_TYPE spi_dev_nrf24l01;
    
        uint8_t rx_aw;
    
        uint8_t write_reg(uint8_t reg,uint8_t value);
        uint8_t read_reg(uint8_t reg);
        uint8_t write_buf(uint8_t reg,uint8_t *buf,uint8_t length);
        uint8_t read_buf (uint8_t reg,uint8_t *buf,uint8_t length);

        void    set_rf_frq(uint8_t frq);
    
        void    set_chanal(uint8_t ch,uint8_t enable);
        void    set_chanal_ack(uint8_t ch,uint8_t enable);
    
        void    set_addr_width(uint8_t aw);
        void    set_local_addr(uint8_t ch,uint8_t *addr);
        void    set_destination_addr(uint8_t *addr);
    
        void    set_pload_width(uint8_t ch,uint8_t width);
        
        void    set_retry_gap(uint8_t gap);
        void    set_retry(uint8_t times);
        
        void    set_gain(uint8_t rf_gain);
        void    set_baudrate(bool baudrate);
        
        void    set_power(bool pwr);
        void    set_tx_rx_mode(bool mode);
        
        void    set_crc(bool crco,bool enable);
    
};
//enum{
//    N18 = 0,N12,N6,N0
//}RF_GAIN;
#define RF_N_18DB 0
#define RF_N_12DB 1
#define RF_N_6DB  2
#define RF_N_0DB  3

#define _1MBPS 0
#define _2MBPS 1

#define TX_MODE 0
#define RX_MODE 1

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//NRF24L01Ĵ
#define NRF_READ_REG    0x00  //üĴ,5λΪĴַ
#define NRF_WRITE_REG   0x20  //дüĴ,5λΪĴַ
#define RD_RX_PLOAD     0x61  //RXЧ,1~32ֽ
#define WR_TX_PLOAD     0xA0  //дTXЧ,1~32ֽ
#define FLUSH_TX        0xE1  //TX FIFOĴ.ģʽ
#define FLUSH_RX        0xE2  //RX FIFOĴ.ģʽ
#define REUSE_TX_PL     0xE3  //ʹһ,CEΪ,ݰϷ.
#define NOP             0xFF  //ղ,״̬Ĵ	 
//SPI(NRF24L01)Ĵַ
#define CONFIG          0x00  //üĴַ;bit0:1ģʽ,0ģʽ;bit1:ѡ;bit2:CRCģʽ;bit3:CRCʹ;
                              //bit4:жMAX_RT(ﵽطж)ʹ;bit5:жTX_DSʹ;bit6:жRX_DRʹ
#define EN_AA           0x01  //ʹԶӦ  bit0~5,Ӧͨ0~5
#define EN_RXADDR       0x02  //յַ,bit0~5,Ӧͨ0~5
#define SETUP_AW        0x03  //õַ(ͨ):bit1,0:00,3ֽ;01,4ֽ;02,5ֽ;
#define SETUP_RETR      0x04  //Զط;bit3:0,Զط;bit7:4,Զطʱ 250*x+86us
#define RF_CH           0x05  //RFͨ,bit6:0,ͨƵ;
#define RF_SETUP        0x06  //RFĴ;bit3:(0:1Mbps,1:2Mbps);bit2:1,书;bit0:Ŵ
#define STATUS          0x07  //״̬Ĵ;bit0:TX FIFO־;bit3:1,ͨ(:6);bit4,ﵽط
                              //bit5:ݷж;bit6:ж;
#define MAX_TX  	0x10  //ﵽʹж
#define TX_OK   	0x20  //TXж
#define RX_OK   	0x40  //յж

#define OBSERVE_TX      0x08  //ͼĴ,bit7:4,ݰʧ;bit3:0,ط
#define CD              0x09  //زĴ,bit0,ز;
#define RX_ADDR_P0      0x0A  //ͨ0յַ,󳤶5ֽ,ֽǰ
#define RX_ADDR_P1      0x0B  //ͨ1յַ,󳤶5ֽ,ֽǰ
#define RX_ADDR_P2      0x0C  //ͨ2յַ,ֽڿ,ֽ,ͬRX_ADDR_P1[39:8];
#define RX_ADDR_P3      0x0D  //ͨ3յַ,ֽڿ,ֽ,ͬRX_ADDR_P1[39:8];
#define RX_ADDR_P4      0x0E  //ͨ4յַ,ֽڿ,ֽ,ͬRX_ADDR_P1[39:8];
#define RX_ADDR_P5      0x0F  //ͨ5յַ,ֽڿ,ֽ,ͬRX_ADDR_P1[39:8];
#define TX_ADDR         0x10  //͵ַ(ֽǰ),ShockBurstTMģʽ,RX_ADDR_P0˵ַ
#define RX_PW_P0        0x11  //ͨ0Чݿ(1~32ֽ),Ϊ0Ƿ
#define RX_PW_P1        0x12  //ͨ1Чݿ(1~32ֽ),Ϊ0Ƿ
#define RX_PW_P2        0x13  //ͨ2Чݿ(1~32ֽ),Ϊ0Ƿ
#define RX_PW_P3        0x14  //ͨ3Чݿ(1~32ֽ),Ϊ0Ƿ
#define RX_PW_P4        0x15  //ͨ4Чݿ(1~32ֽ),Ϊ0Ƿ
#define RX_PW_P5        0x16  //ͨ5Чݿ(1~32ֽ),Ϊ0Ƿ
#define FIFO_STATUS     0x17  //FIFO״̬Ĵ;bit0,RX FIFOĴձ־;bit1,RX FIFO־;bit2,3,
                              //bit4,TX FIFOձ־;bit5,TX FIFO־;bit6,1,ѭһݰ.0,ѭ;
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//configĴλ
//#define RESERVED        0x80
//#define MASK_RX_DR      0x40
//#define MASK_TX_DS      0x20
//#define MASK_MAX_RT     0x10
//#define EN_CRC          0x08
//#define CRCO            0x04
//#define PWR_UP          0x02
//#define PRIM_RX         0x01
//interrupt flag
//#define RX_DR_IT_FLAG   0X40
//#define TX_DS_IT_FLAG   0X40
//#define MAX_RT_IT_FLAG   0X40
//#define RX_DR_IT_FLAG   0X40
//#define RX_DR_IT_FLAG   0X40
//#define RX_DR_IT_FLAG   0X40
//#define TX_FULL_IT_FLAG   0X40

#define TX_ADR_WIDTH    5   //5ֽڵĵַ
#define RX_ADR_WIDTH    5   //5ֽڵĵַ
#define TX_PLOAD_WIDTH  4  //20ֽڵûݿ
#define RX_PLOAD_WIDTH  4  //20ֽڵûݿ

#endif
