/*
file   : *.cpp
author : link
version: V1.0
date   : 2016/15/5

*/

//STM32 RUN IN eBox


#include "ebox.h"
#include "Max7456.h"
#include "OSD_Panels.h"
#include "ringbuf.h"
#include "can.h"
#include "NazaCanDecoderLib.h"
#include "hw_config.h"
#include "usb_wrapper.h"
#include "setting.h"
#include "util.h"

#define UART_RX_BUFFER_MAX 64
uint8_t uart_buf[UART_RX_BUFFER_MAX];


OSD osd(&PA4, &spi1);
RINGBUF uart_rx_buf;
CAN can(&PB8, &PB9);
PWM camera(&PA2);

// May be moved in GlobalVariables.h


unsigned long previous_millis_low = 0;
unsigned long previous_millis_high = 0;
int hi_speed_cycle = 50;
int lo_speed_cycle = 100;
uint32_t messageId;

extern volatile  uint8_t usb_can_flag;

extern float        osd_vbat_A ;
extern int16_t      osd_battery_remaining_A ;
extern int8_t       osd_pitch;
extern int8_t       osd_roll;
extern int8_t 			osd_mode;
extern float        osd_lat;                    // latidude
extern float        osd_lon;                    // longitude
extern uint8_t      osd_satellites_visible;
extern float        osd_climb;
extern float        osd_airspeed;
extern uint8_t      osd_fix_type;               // GPS lock 0-1=no fix, 2=2D, 3=3D
extern float        osd_alt;
extern float        osd_heading;
extern uint16_t     	osd_rc_raw;

void uart_rx_interrupt_event(void)
{
    uint8_t c;
    c = uart1.receive();
    uart_rx_buf.write(c);
}



void setup()
{
    ebox_init();

    LED.mode(OUTPUT_PP);
    USB_DET.mode(INPUT_PD);
		camera.begin(50,75);
		
    uart1.begin(57600);
    uart_rx_buf.begin(uart_buf, UART_RX_BUFFER_MAX);
    uart1.attach_rx_interrupt(uart_rx_interrupt_event);
	
		delay_ms(50);//max7456 power up need 50ms for statable
		osd.begin(1);
		osd.init();
		osd.clear(); 
	
    load_flash_setting();

    if(USB_DET.read() != 0)
    {       
        usb_can_flag = 1;
        LED.reset();
        enter_usb_mode();
				OSD_test_display();
        usb_updata_font();
    }
		
		startPanels();
		delay_ms(2000);
		osd.clear(); 
		
    usb_can_flag = 0;
    NazaCanDecoder.begin();
		
}
int main(void)
{
    setup();

    //while(1);
    //check_update_fontmode();
    while(1)
    {
        //---------------  Start Timed Service Routines  ---------------------------------------
        unsigned long currentMillis = millis();

        if((currentMillis - previous_millis_low) >= lo_speed_cycle)  // 10 Hz (Executed every 100ms)
        {
            previous_millis_low = currentMillis;
            if(on_time.on_off) panTime(on_time.x, on_time.y);
            setHomeVars();   // calculate and set Distance from home and Direction to home
            if(home_direction.on_off) panHomeDir(home_direction.x, home_direction.y);
            if(home_distance.on_off) panHomeDis(home_distance.x, home_distance.y);
            if(home_altitude.on_off) panHomeAlt(home_altitude.x, home_altitude.y);
            if(heading.on_off) panHeading(heading.x, heading.y);
            LED.toggle();
        }  // End of slow Timed Service Routine (100ms loop)

        if((currentMillis - previous_millis_high) >= hi_speed_cycle)  // 20 Hz (Executed every 50ms)
        {
            previous_millis_high = currentMillis;


        }	 // End of fast Timed Service Routine (50ms loop)

        messageId = NazaCanDecoder.decode();
        if(messageId == NAZA_MESSAGE_MSG0926)
        {
            //uart1.printf("bat=%d\r\n",NazaCanDecoder.getBattery());
            //uart1.printf("percent=%d\r\n",NazaCanDecoder.getBatteryPercent());
            osd_vbat_A = (float) NazaCanDecoder.getBattery() / 1000;
            osd_battery_remaining_A = NazaCanDecoder.getBatteryPercent();
            if(battary_voltage.on_off) panBatt_A(battary_voltage.x, battary_voltage.y);
            if(battary_percent.on_off) panBatteryPercent(battary_percent.x, battary_percent.y);
        }
        else if (messageId == NAZA_MESSAGE_MSG1009)
        {
            osd_pitch = NazaCanDecoder.getPitch();
            osd_roll = NazaCanDecoder.getRoll();
            osd_mode = NazaCanDecoder.getMode();
            if(pitch.on_off) panPitch(pitch.x, pitch.y);
            if(roll.on_off) panRoll(roll.x, roll.y);
            if(flight_mode.on_off) panFlightMode(flight_mode.x, flight_mode.y);
            if(horizonr.on_off) panHorizon(horizonr.x, horizonr.y);
            //uart1.printf("Pitch:%d ",osd_pitch);
            //uart1.printf(", Roll:%d \r\n",osd_roll);
        }
        else if (messageId == NAZA_MESSAGE_MSG1002)
        {
            osd_lat = NazaCanDecoder.getLat();
            osd_lon = NazaCanDecoder.getLon();
            osd_satellites_visible = NazaCanDecoder.getNumSat();
            osd_climb = NazaCanDecoder.getVsi();
            osd_airspeed = NazaCanDecoder.getSpeed();
						osd_alt =  NazaCanDecoder.getAlt();
						osd_heading = NazaCanDecoder.getHeadingNc();
						osd_rc_raw = (uint16_t)map(constrain(NazaCanDecoder.getRcIn(NazaCanDecoderLib::RC_X1), -1000, 1000), -1000, 1000, 1000, 2000);
						camera.set_duty((uint16_t)map(osd_rc_raw,1000,2000,50,100));
					  if(GPS_number.on_off) panGPSats(GPS_number.x, GPS_number.y);
            if(gps_value.on_off) panGPS(gps_value.x, gps_value.y);
            if(clamb_speed.on_off) panClimb(clamb_speed.x, clamb_speed.y);
            if(airspeed.on_off) panAirSpeed(airspeed.x, airspeed.y);
        }
				else if (messageId == NAZA_MESSAGE_MSG1003)
				{
						osd_fix_type = NazaCanDecoder.getFixType();
				}
        //NazaCanDecoder.heartbeat();

    }
}



